# This is the access authorisation module for the NRH-up2date
# project. It will allow authorised clients to download content,
# according to variables set in apache. Sample httpd.com config :
#
#    PerlModule My::Access
#
#    <Directory "/var/spool/nrh-up2date/">
#        <FilesMatch "\.rpm$">
#            PerlAccessHandler My::Access
#            PerlSetVar Intranet "10.0.0.1 => Comp1, 10.0.0.2 => Comp2"
#            PerlSetVar security 1
#            PerlSetVar secret "MySecret"
#        </FilesMatch>
#    </Directory>
#
# In my setup, the module is placed at the following location :
# /usr/lib/perl5/vendor_perl/5.6.1/i386-linux/My/Access.pm
#
# The Intranet hash will set the ip addresses that will have unrestricted
# access to the protected location. Setting security to 1 will allow only
# system with systemid's valid to your server be able to download content,
# and 0 means that any up2date client that connected to the server can
# download content. Secret must contain the same text that "/etc/nrh/secret"
# contains, or autorization will not work at all.
#
# This module has been released with version 1.1 of NRH-up2date.

package My::Access;
use Apache::Constants;
use Digest::MD5;

use strict;

sub handler {
    my $r = shift;

    my %ips = split /\s*(?:=>|,)\s*/, $r->dir_config("Intranet");
  
    if (my $user = $ips{$r->connection->remote_ip}) {
  
        $r->log_reason("Logging on $user");
	# update connection record
        $r->connection->user($user);
  
        # do not ask for a password
        $r->set_handlers(PerlAuthenHandler => [\&OK]);
	return OK;
    }

    my $authtoken = $r->header_in('X-RHN-Auth');
    if (($authtoken eq 'anonymous') && ($r->dir_config('security') == 0)) {
        $r->warn('Logging on anonymous user');
        # update connection record
        $r->connection->user('anonymous');

        # do not ask for a password
        $r->set_handlers(PerlAuthenHandler => [\&OK]);
        return OK;
    }

    my $username = $r->header_in('X-RHN-Auth-User-Id');
    if ($username eq 'anonymous') {
	my $md5;
        $md5 = Digest::MD5->new;
        $md5->add($username);
        $md5->add($r->dir_config('secret'));
        if ($authtoken eq $md5->b64digest) {
            $r->warn("Logging on registered user");
            # update connection record
            $r->connection->user('anonymous');

            # do not ask for a password
            $r->set_handlers(PerlAuthenHandler => [\&OK]);
            return OK;
        }
    }

    $r->log_reason("No known user");
    return FORBIDDEN;
}
1;
